﻿Type=StaticCode
Version=4
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
Sub Process_Globals
	Dim playList As List				' List and order of tracks to be played
	Dim playHistory As List				' List and order of tracks already played
	Type trackInfo( key As String, _
					clvIndex As Int, _
					title As String, _
					location As String)	' Details of each track
End Sub

Public Sub ClearTracks()
	' Clear playlist and reset selected status in browsed results
	If playList.IsInitialized Then
		For Each track As trackInfo In playList
			' Remove selected status in BrowseResults and de-select in CustomListView
			clvSelectionUpdate(track, False)
		Next
		' Remove all entries in playList
		playList.Clear
	End If
End Sub

Public Sub RemoveTrack(track As trackInfo)
	' Check that track is initialised (i.e. contains track info)
	If track.IsInitialized Then
		' Get track position in playlist
		Dim trackPosition As Int
		trackPosition = playList.IndexOf(track)
		' Check if currently playing
		If trackPosition = 0 Then
			' When in looping mode do not remove track and so it will be played again!
			If Music_Service.blnlooping = False Then
				' Track currently playing, stop, remove from playlist and autostart next track
				Music_Service.musicStream.Stop
				' Initialise playHistory if required and add track
				If playHistory.IsInitialized = False Then playHistory.Initialize
				playHistory.InsertAt(0, playList.get(0))
				playList.RemoveAt(0)
			End If
			CallSub(Music_Service, "musicStream_Load")
		Else
			' Remove track from playlist
			playList.RemoveAt(trackPosition)
		End If
		' Deselect the track except for when in continuous looping mode
		If Music_Service.blnlooping = False Then clvSelectionUpdate(track, False)
	End If
End Sub

Public Sub GetTrackWithKey(key As String) As trackInfo
	' Return the required track if in playlist
	For Each track As trackInfo In playList
		If track.key = key Then Return track
	Next
	' Return an un-initialised track if not found
	Dim track As trackInfo
	Return track
End Sub

Public Sub SelectTrack(key As String, clvIndex As Int, toggle As Boolean)
	' Short click to select, long click to toggle (triggered by clv click event)
	Dim container As itemContainer
	container.Initialize
	container = UPNP.browseResults.get(key)	
	' Determine if to select or toggle item
	If toggle = True Then
		' Get current selection status and toggle
		container.selected = Not(container.selected)
	Else
		' Short click selects item
		container.selected = True
	End If
	' Update selection status
	UPNP.browseResults.Put(key, container)
	If container.selected = True Then
		Debug.debugLog("##### SELECTED AUDIO FILE DETAILS  #####" & CRLF & _
					   "      [id         =" & key & "]" & CRLF & _
					   "      [title      =" & container.title & "]" & CRLF & _
					   "      [class      =" & container.itemClass & "]" & CRLF & _
					   "      [location   =" & container.location & "]") 'DEBUG
		' Add track to playlist
		LoadTrack(key, clvIndex, container.title, container.location, toggle)
	Else
		' Remove track from playlist
		RemoveTrack(GetTrackWithKey(key))
	End If
End Sub

Public Sub CurrentlyPlayingTrack As trackInfo
	' Return info about the currently playing track
	Return playList.get(0)
End Sub

Public Sub PlayNextTrack()
	' Called by the user selecting next track button on the audio controls panel
	CallSub(Music_Service, "musicStream_Complete")
End Sub

Public Sub PlayPreviousTrack()
	' Called by the user selecting previous track button on the audio controls panel
	If playHistory.IsInitialized Then
		' Check play history to see if a previous track exists
		If playHistory.Size > 0 Then
			' Stop the currently playing track and select the previous track
			Music_Service.musicStream.Stop
			clvSelectionUpdate(playHistory.get(0), True)
			' Remove track from play history and add back into playlist
			playList.InsertAt(0, playHistory.get(0))
			playHistory.RemoveAt(0)
			' Load the track to start playing
			CallSub(Music_Service, "musicStream_Load")
		End If
	End If
End Sub

Public Sub LoadTrack(key As String, clvIndex As Int, title As String, location As String, addToList As Boolean)
	' Create track info for playlist
	Dim track As trackInfo
	track.Initialize
	track.key = key
	track.clvIndex = clvIndex
	track.title = title
	track.location = location
	' Initialise play list if starting a new list
	If playList.IsInitialized = False OR addToList = False Then playList.Initialize
	playList.Add(track)
	' Start music playback if first track in list
	If playList.Size = 1 Then StartService(Music_Service)
	' Update CustomListView
	clvSelectionUpdate(track, True)
End Sub

Public Sub clvSelectionUpdate(track As trackInfo, selected As Boolean)
	' Check if track present in currently browsed results (user may have navigated away!)
	If UPNP.browseResults.ContainsKey(track.key) Then
		Dim container As itemContainer
		container.Initialize
		' Get item and update selected status
		container = UPNP.browseResults.get(track.key)
		container.selected = selected
		UPNP.browseResults.Put(track.key, container)
	End If
	' Update CustomListView if currently active (will be updated on resume if not currently active)
	If IsPaused(Main) = False Then
		CallSubDelayed3(Main, "clvItem_Selection", track.clvIndex, selected)
	End If
End Sub