﻿Type=Service
Version=4
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
#Region  Service Attributes 
	#StartAtBoot: False
#End Region

Sub Process_Globals
	Dim musicStream As MediaPlayerStream
	' Player status variables
	Dim playerInitialised, blnlooping As Boolean
	Dim playerStatus, currentlyPlaying As String
	Dim statusTimer As Timer
	' System variables
	Dim n As Notification
	Private wifiLock As Object
	Dim pws As PhoneWakeState
End Sub

Public Sub Service_Create
	' Initialise audio stream and set status to idle
	musicStream.Initialize("musicStream")
	playerInitialised = True
	playerStatus = "idle"
	statusTimer.Initialize("statusTimer", 3000)
	' Set WiFi lock so that wireless doesn't attempt switch off (power save)
	wifiLock_Initialise
End Sub

Public Sub Service_Start (StartingIntent As Intent)
	' Create app notification to ensure it is kept alive
	notification_Initialise
	' Acquire a partial lock to prevent device from entering sleep mode
	pws.PartialLock
	' Acquire a WiFi lock to prevent the wireless from switching off
	wifiLockState("acquire")
	' Load the first track in playlist
	musicStream_Load
End Sub

Public Sub Service_Destroy
	' Stop the service and notification, release partial lock and WiFi lock
	Service.StopForeground(1)
	pws.ReleasePartialLock
	wifiLockState("release")
End Sub

Private Sub notification_Initialise()
	' App notification
	n.Initialize
	n.Icon = "icon"
	n.SetInfo("UPNP Browser", "", Main) 
	n.Sound=False
	n.Vibrate=False
	n.Light=False
	n.OnGoingEvent=True
	' Set this service to run in the foreground
	Service.StartForeground(1,n)
End Sub

Private Sub wifiLock_Initialise()
	' Create a WiFi lock object using reflector library
	Dim r As Reflector
	r.Target = r.GetContext
	r.Target = r.RunMethod2("getSystemService", "wifi", "java.lang.String")
	wifiLock = r.RunMethod4("createWifiLock", Array As Object(1, "locktag"), _
								Array As String("java.lang.int", "java.lang.String"))
End Sub

Private Sub wifiLockState(lockState As String)
	' Control WiFi lock (accepted values are "acquire" and "release")
   Dim r As Reflector
   r.Target = wifiLock
   r.RunMethod(lockState)
End Sub

Public Sub musicStream_Load
	' Load the next available track or stop service if no more tracks in playlist
	If Music.playList.Size > 0 Then
		' Load next track
		Dim track As trackInfo
		track.Initialize
		track = Music.playList.Get(0)
		musicStream.Load(track.location)
		' Track info for notification text
		currentlyPlaying = track.title
	Else
		' No more tracks, hide audio controls and stop service
		CallSub2(Main, "audioControl_Display", False)
		currentlyPlaying = ""
		Service.StopForeground(1)
	End If
End Sub

Private Sub musicStream_StreamReady
	' StreamReady event triggered after loading a track and it becoming ready to play
	' Play track and update player status
    musicStream.Play
	playerStatus = "playing"
    Debug.debugLog( "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~" & CRLF & _
					"NOW PLAYING :- " & Music.CurrentlyPlayingTrack.title & CRLF & _
					"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~")
	' Display audio controls and start timer for track info
	CallSub2(Main, "audioControl_Display", True)
	statusTimer.Enabled = True
End Sub

Private Sub musicStream_StreamError (ErrorCode As String, ExtraData As Int)
	' StreamError event triggered after loading a track if it is unavailable for any reason
    Debug.debugLog("MediaPlayerStream error: " & ErrorCode & ", " & ExtraData)
    ToastMessageShow("Error: " & ErrorCode & ", " & ExtraData, True)
End Sub

Public Sub musicStream_Complete
	' Complete event triggered when track has ended
	' Stop timer for track info and update player status
	statusTimer.Enabled = False
	playerStatus = "idle"
	' Remove the track from the playlist (this will be added to the play history list)
	Music.RemoveTrack(Music.playList.Get(0))
End Sub

Private Sub statusTimer_Tick()
	' Track info for notification text
	getPosition
End Sub

Public Sub getPosition()
	' Use java object to enquire about the current position of playing track
	Dim jo As JavaObject = musicStream
	Dim position As Int
	position = jo.RunMethod("getPosition", Null)
	' Get track position and duration, format as "m:ss / m:ss"
	Dim duration As String = ConvertToTimeFormat(position) & " / " & ConvertToTimeFormat(musicStream.duration)
	' Update log and app notification with track title and runtime info 
	Debug.debugLog(duration) 'DEBUG
	n.SetInfo(currentlyPlaying, duration, "")
	n.Notify(1)
End Sub

Sub ConvertToTimeFormat(ms As Int) As String
	' Format ms into minutes and seconds
    Dim seconds, minutes As Int
    seconds = Round(ms / 1000)
    minutes = Floor(seconds / 60)
    seconds = seconds Mod 60
	' Return in time format "m:ss"
    Return NumberFormat(minutes, 1, 0) & ":" & NumberFormat(seconds, 2, 0)
End Sub
