﻿Type=StaticCode
Version=4
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
Public Sub Process_Globals
End Sub

'Performs a case insensitive search for pattern in text string
'Pattern - A valid regular expression
'Text - String to search
'
'Returns group 1 of ReEx result or "" if not found
'Example: <code>
'Dim uuid As String
'	uuid=Extract("usn:[\s]?([\S]+)", "USN:uuid:156ad7e9-2736-44b8-8628-eec1b836cb1e:MediaServer:1)"</code>
Public Sub Extract( Pattern As String, Text As String) As String
	Dim m As Matcher
	m = Regex.Matcher2(Pattern, Regex.CASE_INSENSITIVE, Text)
	If m.Find = True Then Return m.Group(1)
	' Return an empty string if pattern not found
	Return ""
End Sub

'Strips two numbers from a string
'Numbers can be seperated with any none numeric charater
'strRes - Any string containing two numbers eg. "1024x576"
'
'Returns a type Resolution containing x and y values
'Example: <code>
'vidResolution = RegExp.ExtractResolution("1024x576)</code>
Public Sub ExtractResolution(strRes As String) As Resolution
	Dim i As Int
	Dim x, y As String = ""
	Dim firstNum As Boolean = True
	' Loop through each character in the string
	For i = 0 To strRes.Length - 1
		If firstNum = True Then
			' First number extraction
			If IsNumber(strRes.CharAt(i)) Then
				' Update x if there is a numeric charater at the present position 
				x = x & strRes.CharAt(i)
			Else If x <> "" Then
				' Exit if a number has been obtained and present position is no longer a numeric character
				firstNum = False
			End If		
		Else
			' Second number extraction
			If IsNumber(strRes.CharAt(i)) Then
				' Update y if there is a numeric charater at the present position 
				y = y & strRes.CharAt(i)
			Else If y <> "" Then
				' Exit if a number has been obtained and present position is no longer a numeric character
				Exit
			End If	
		End If
	Next
	' Check that two numbers have been obtained, string may have been invalid
	If x = "" OR y = "" Then
		' Default values in case of invalid string
		x = 0
		y = 0
	End If
	' Return video resolution values
	Dim res As Resolution
	res.Initialize
	res.x = x
	res.y = y
	Debug.debugLog("Resolution=" & strRes & "  ResX=" & x & "  ResY=" & y)
	Return res
End Sub