﻿Type=Service
Version=4
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
#Region  Service Attributes 
	#StartAtBoot: False
#End Region

Public Sub Process_Globals
    Dim udpSocket1 As UDPSocket
    Dim serverResponse, browseURL, browseServiceType, browseIDResume As String
	Dim browseID, browseFilter, browseStartingIndex, browseRequestedCount, browseSortCriteria As String
	Dim browseReset, browseAbort, browseMetaDataResume As Boolean
	Dim eotTimer As Timer
	Dim serverID As Map
	Type UPNPserver(rootURL As String, _
					descriptionURL As String, _
					name As String, _
					modelDescription As String, _
					iconURL As String, _
					iconBitmap As Bitmap, _
					contentType As String, _
					contentSCPDURL As String, _
					contentControlURL As String, _
					contentEventSubURL As String, _
					connectionType As String, _
					connectionSCPDURL As String, _
					connectionControlURL As String, _
					connectionEventSubURL As String)
	Dim ctrlParser As Ctrl_Parser
	Dim browseParser As Browse_Parser
	' The following globals are populated by the browseParser Class Module
	Dim browseLimit As Int
	Dim browseResults As Map
	Type itemContainer(title As String, _
				       itemClass As String, _
					   location As String, _
					   strResolution As String, _
					   selected As Boolean) ' Other information can be added to container
End Sub

Public Sub Service_Create
	' Initialise port and server map then search for UPNP MediaServers
	Initialise
	M_Search(2, "urn:schemas-upnp-org:device:MediaServer:1")
	' Set time-out to 5s
	EOT_Delay_Reset(5000)
End Sub

#Region -------------------------[ UDP M_Search ]-------------------------
Private Sub Initialise
	' Initialise UDP socket on first available port
	udpSocket1.Initialize("UDP", 0, 8000)
	' Initialse Map (Key=server uuid As String , Value=server details As UPNPserver)
	serverID.Initialize
	eotTimer.Initialize("EOT_Delay", 5000)
	serverResponse = ""
End Sub

'Send a multicast message to search for devices of device type defined by 'st'
'mx - Maximum wait time in seconds
'st - Search target as defined by the UPNP device architecture
'
'Example: <code>
'M_Search(3, "ssdp:all")</code>
Private Sub M_Search(mx As Int, st As String)
	' Compile HTTP broadcast message
	Dim msg As String		
	msg = "M-SEARCH * HTTP/1.1~" & _
		  "HOST: 239.255.255.250:1900~" & _
    	  "MAN: " & Chr(34) & "ssdp:discover" & Chr(34) & "~" & _
		  "MX: " & mx & "~" & _
		  "ST: " & st & "~~"
	' Replace "~" with "\r\n" [NOTE:- CRLF only sends Chr(13) eg."\r"]
	msg = msg.Replace("~", Chr(13) & Chr(10))
	Debug.debugLog(msg) 'DEBUG
	' Convert string to UTF8
    Dim data() As Byte
    data = msg.GetBytes("UTF8")
	' Initialise packet and send
	Dim packet As UDPPacket
    packet.Initialize(data, "239.255.255.250", 1900)
    udpSocket1.Send(packet)
End Sub

Private Sub UDP_PacketArrived (Packet As UDPPacket)
	' Restart 1.5s timer each time a new packet arrives
	EOT_Delay_Reset(1500)
	' Convert received data to UTF8
    serverResponse = BytesToString(Packet.Data, Packet.Offset, Packet.Length, "UTF8")
	Debug.debugLog(CRLF & serverResponse) 'DEBUG
	' Extract device uuid and description file url
	Get_Details
End Sub

Private Sub EOT_Delay_Reset(ms As Int)
	' Restart End-Of-Transmission timer with an interval of ms
	' Purpose is to detect end of device descovery
	eotTimer.Enabled = False
	eotTimer.Interval = ms
	eotTimer.Enabled = True
End Sub

Private Sub EOT_Delay_Tick()
	' Stop End-Of-Transmission timer and close UDP socket
	eotTimer.Enabled = False
	ProgressDialogHide
	udpSocket1.Close
	Debug.debugLog("##### All UPNP servers responded #####") 'DEBUG
	' Display discovered devices or error message if no response received
	If serverID.Size > 0 Then
		List_Servers
	Else
		CallSubDelayed3(Main, "ShowMsgBox", "Server(s) not responding" & CRLF & "Check wifi is turned on and rescan", "Network Error...")
		ProgressDialogHide
	End If
End Sub

Private Sub Get_Details
	' Check serverResponse is from a MediaServer
	If serverResponse.Contains("urn:schemas-upnp-org:device:MediaServer:1") Then
		' Attempt to extract server details
		Dim uuid, descriptionUrl, rootUrl As String
		uuid = RegExp.Extract("usn:[\s]?([\S]+)", serverResponse)
		descriptionUrl = RegExp.Extract("location:[\s]?([\S]+)", serverResponse)
		rootUrl = RegExp.Extract("(http://(\d{1,3}.){3}\d{1,3}:\d+)", descriptionUrl)
		Debug.debugLog("uuid     = " & uuid & CRLF & _
					   "location = " & descriptionUrl & CRLF & _ 
					   "rootUrl  = " & rootUrl & CRLF & _
					   "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~" ) 'DEBUG
		' Add to list of media servers if all details present
		If uuid <> "" AND descriptionUrl <> "" AND rootUrl <> "" Then
			' Map server details
			Dim server As UPNPserver
			server.rootUrl = rootUrl
			server.descriptionUrl = descriptionUrl
			serverID.Put(uuid, server)
		End If
	End If
End Sub
#End Region

#Region -------------------------[ Device Discovery ]------------------------- 
Private Sub List_Servers()
	' Get details for each server
	Dim i As Int
	For i = 0 To serverID.Size - 1
		' Use server uuid to identify each job and prefix with "Description "
		Dim httpReq As HttpJob
		httpReq.Initialize("Description " & serverID.GetKeyAt(i), Me)
		' Get server description file location
		Dim server As UPNPserver=serverID.GetValueAt(i)
		httpReq.Download(server.descriptionURL)
		Debug.debugLog("HTTP for index... " & i & " = " & server.descriptionURL) 'DEBUG
	Next
	CallSubDelayed2(Main, "LoadCustomListView", Main.clvJumpPosition)
	Debug.debugLog("##### SERVER LIST DISPLAYED #####") 'DEBUG
End Sub

Public Sub Request_Icon(iconUrl As String, uuid As String)
	' Use server uuid to identify each job and prefix with "Icon "
	Dim httpReq As HttpJob
	httpReq.Initialize("Icon " & uuid, Me)
	' Get icon file
	httpReq.Download(iconUrl)
	Debug.debugLog("HTTP for icon... " & uuid & " = " & iconUrl) 'DEBUG
End Sub
#End Region

#Region -------------------------[ Device Control Actions]------------------------- 
Public Sub Request_Controls(uuid As String)
	' Get url of service control protocol description for device
	Dim server As UPNPserver
	server = serverID.Get(uuid)
	' Use server uuid to identify each job and prefix with "Control "
	Dim httpReq As HttpJob
	httpReq.Initialize("Control " & uuid, Me)
	' Get control XML file
	httpReq.Download(server.rootURL & server.contentSCPDURL)
	Debug.debugLog("HTTP for control XML... " & uuid & " = " & server.rootURL & server.contentSCPDURL) 'DEBUG
End Sub
#End Region

#Region -------------------------[ Browse Action ]------------------------- 
Public Sub CheckBrowseAvailability(uuid As String)
	' Browse server only if Browse action was present in the control XML file
	If ctrlParser.serverActions.IndexOf("Browse") >= 0 Then
		Dim server As UPNPserver = serverID.Get(uuid)
		browseURL = server.rootURL & server.contentControlURL
		browseServiceType = server.contentType
		browseFilter = "*"
		browseStartingIndex = 0
		browseRequestedCount = 100
		browseSortCriteria = ""
		browseReset = True
		BrowseServer(0, False)
	End If
End Sub

Private Sub BrowseServer(objectID As String, metaData As Boolean)
	' Do not browse if abort flag is set
	If Not(browseAbort) Then
		' Reset browse resume ID
		browseIDResume = ""
		Debug.debugLog("##### SEND BROWSE REQUEST #####") 'DEBUG
		' Set browseFlag to either BrowseMetadata or BrowseDirectChildren
		Dim browseFlag As String
		If metaData = True Then
			browseFlag = "BrowseMetadata"
		Else
			browseFlag = "BrowseDirectChildren"
		End If

		' Format browse ID to correct HTML entity
		browseID = Format.EntitiesHTML(objectID)
		' Create SOAP XML message
		Dim xmlMsg As String
		xmlMsg = "<?xml version='1.0' encoding='utf-8'?>" & _
				 "<s:Envelope xmlns:s='http://schemas.xmlsoap.org/soap/envelope/' s:encodingStyle='http://schemas.xmlsoap.org/soap/encoding/'>" & _
					"<s:Body>" & _
						"<u:Browse xmlns:u='" & browseServiceType & "'>" & _
							"<ObjectID>" & browseID & "</ObjectID>" & _
							"<BrowseFlag>" & browseFlag & "</BrowseFlag>" & _
							"<Filter>" & browseFilter & "</Filter>" & _
							"<StartingIndex>" & browseStartingIndex & "</StartingIndex>" & _
							"<RequestedCount>" & browseRequestedCount & "</RequestedCount>" & _
							"<SortCriteria>" & browseSortCriteria & "</SortCriteria>" & _
						"</u:Browse>" & _
					"</s:Body>" & _
				  "</s:Envelope>"
		xmlMsg = xmlMsg.Replace("'", Chr(34))
		Debug.debugLog(xmlMsg) 'DEBUG

		' Initialise HTTP job
		Dim httpReq As HttpJob
		httpReq.Initialize("Browse",Me)
		httpReq.PostString(browseURL, xmlMsg)
		httpReq.GetRequest.SetContentType("text/xml; charset=" & Chr(34) & "utf-8" & Chr(34))
		httpReq.GetRequest.SetHeader("SOAPAction",Chr(34) & browseServiceType & "#Browse" & Chr(34))
	    httpReq.GetRequest.Timeout = 20000
		If browseReset = True Then
			' Store the objectID being browsed (enables navigating back through browsed items)
			Main.browseLog.Add(objectID)
		End If
	Else
		' Reset browse abort flag, hide progress bar and store browse variables in case of resume
		CallSub2(Main, "progress_Update", False)
		browseAbort = False
		browseIDResume = objectID
		browseMetaDataResume = metaData
	End If
End Sub
#End Region

#Region -------------------------[ HTTP Responses ]-------------------------
Private Sub JobDone (job As HttpJob)
	' Event raised when httpReq download completed or failed
	Debug.debugLog("JobName = " & job.JobName & ", Success = " & job.Success) 'DEBUG
	If job.Success = True Then
		' Determine which information was requested, jobId = server uuid
		Dim jobType As String=RegExp.Extract("(\w+)",job.JobName)
		Dim jobId As String=RegExp.Extract("[\s]{1}([\S]+)",job.JobName)
		Select Case jobType
			Case "Description"
				Debug.debugLog("##### DEVICE DESCRIPTION XML FILE #####") 'DEBUG
				Debug.debugLog(job.GetString) 'DEBUG
				' Parse XML to find server details
				Dim descParser As Desc_Parser
				descParser.Initialize(job.GetString, jobId)
			Case "Icon"
				Debug.debugLog("##### ICON FILE AQUIRED #####") 'DEBUG
				Dim server As UPNPserver = serverID.Get(jobId)
				' Format icon for CustomListView
				server.iconBitmap = Format.CreateScaledBitmap(job.GetBitmap, 80dip, 80dip, True)
				CallSubDelayed(Main, "UpdateEndItem")
				CallSubDelayed2(Main, "LoadCustomListView",Main.clvJumpPosition)
			Case "Control"
				Debug.debugLog("##### DEVICE CONTROL XML FILE #####") 'DEBUG
				Debug.debugLog(job.GetString) 'DEBUG
				' Parse XML to find server controls
				ctrlParser.Initialize(job.GetString, jobId)
			Case "Browse"
				Debug.debugLog("##### BROWSE RESULTS (RAW DATA) #####") 'DEBUG
				Debug.debugLog(job.GetString.SubString2(0,Min(job.GetString.Length-1, 3999))) 'DEBUG
				' Parse XML to get browsed results
				browseParser.Initialize(job.GetString)
			Case Else
				Debug.debugLog("##### UNKNOWN JOB TYPE #####" & CRLF & "Job Type = " & jobType) 'DEBUG
		End Select
	Else
		' Download request failed, notify user
		Debug.debugLog("Error: " & job.ErrorMessage) 'DEBUG
		If job.ErrorMessage = "Internal Server Error" Then
			CallSubDelayed3(Main, "ShowMsgBox", "Please check network and sharing setting on the remote device.", "Access Denied!")
		End If
	End If
	job.Release
End Sub
#End Region